@@ -9,10 +9,16 @@
 #' @param site A site name search string e.g. `site = "heathrow"`. The search
 #'   strings and be partial and can be upper or lower case e.g. `site =
 #'   "HEATHR"`.
-#' @param lat A latitude in decimal degrees to search. Takes the values -90 to
-#'   90.
-#' @param lon A longitude in decimal degrees to search. Takes values -180 to
-#'   180. Negative numbers are west of the Greenwich meridian.
+#' @param lat,lon Decimal latitude and longitude (or other Y/X coordinate if
+#'   using a different `crs`). If provided, the `n` closest ISD stations to this
+#'   coordinate will be returned.
+#' @param crs The coordinate reference system (CRS) of the data, passed to
+#'   [sf::st_crs()]. By default this is [EPSG:4326](https://epsg.io/4326), the
+#'   CRS associated with the commonly used latitude and longitude coordinates.
+#'   Different coordinate systems can be specified using `crs` (e.g., `crs =
+#'   27700` for the [British National Grid](https://epsg.io/27700)). Note that
+#'   non-lat/lng coordinate systems will be re-projected to EPSG:4326 for making
+#'   comparisons with the NOAA metadata plotting on the map.
 #' @param country The country code. This is a two letter code. For a full
 #'   listing see <https://www1.ncdc.noaa.gov/pub/data/noaa/isd-history.csv>.
 #' @param state The state code. This is a two letter code.
@@ -37,11 +43,9 @@
 #'   latitude and longitude searches are made an approximate distance, `dist` in
 #'   km is also returned.
 #' @export
-#' @seealso [getMetaLive()] to download the all meta data to allow re-use and
-#'   direct querying.
 #' @author David Carslaw
+#' @family NOAA ISD functions
 #' @examples
-#'
 #' \dontrun{
 #' ## search for sites with name beijing
 #' getMeta(site = "beijing")
@@ -52,90 +56,101 @@
 #' ## returns 'n' nearest by default
 #' getMeta(lat = 40, lon = 116.9)
 #' }
-getMeta <- function(site = "heathrow",
-                    lat = NA,
-                    lon = NA,
-                    country = NA,
-                    state = NA,
-                    n = 10,
-                    end.year = "current",
-                    provider = c("OpenStreetMap", "Esri.WorldImagery"),
-                    plot = TRUE,
-                    returnMap = FALSE) {
+getMeta <- function(
+  site = "heathrow",
+  lat = NA,
+  lon = NA,
+  crs = 4326,
+  country = NA,
+  state = NA,
+  n = 10,
+  end.year = "current",
+  provider = c("OpenStreetMap", "Esri.WorldImagery"),
+  plot = TRUE,
+  returnMap = FALSE
+) {
   ## read the meta data
-  
+
   ## download the file, else use the package version
   meta <- getMetaLive()
-  
+
   # check year
   if (!any(end.year %in% c("current", "all"))) {
     if (!is.numeric(end.year)) {
-      stop("end.year should be one of 'current', 'all' or a numeric 4-digit year such as 2016.")
+      cli::cli_abort(
+        "{.field end.year} should be one of 'current', 'all' or a numeric 4-digit year such as {2016}."
+      )
     }
   }
-  
+
   # we base the current year as the max available in the meta data
-  if ("current" %in% end.year)
+  if ("current" %in% end.year) {
     end.year <-
       max(as.numeric(format(meta$END, "%Y")), na.rm = TRUE)
-  if ("all" %in% end.year)
+  }
+  if ("all" %in% end.year) {
     end.year <- 1900:2100
-  
-  
-  
+  }
+
   ## search based on name of site
   if (!missing(site)) {
     ## search for station
-    meta <- meta[grep(site, meta$STATION, ignore.case = TRUE),]
+    meta <- meta[grep(site, meta$STATION, ignore.case = TRUE), ]
   }
-  
+
   ## search based on country codes
   if (!missing(country) && !is.na(country)) {
     ## search for country
     id <- which(meta$CTRY %in% toupper(country))
-    meta <- meta[id,]
+    meta <- meta[id, ]
   }
-  
+
   ## search based on state codes
   if (!missing(state)) {
     ## search for state
     id <- which(meta$ST %in% toupper(state))
-    meta <- meta[id,]
+    meta <- meta[id, ]
   }
-  
+
   # make sure no missing lat / lon
   id <- which(is.na(meta$LON))
-  if (length(id) > 0)
-    meta <- meta[-id,]
-  
+
+  if (length(id) > 0) {
+    meta <- meta[-id, ]
+  }
+
   id <- which(is.na(meta$LAT))
-  if (length(id) > 0)
-    meta <- meta[-id,]
-  
+  if (length(id) > 0) {
+    meta <- meta[-id, ]
+  }
+
   # filter by end year
   id <- which(format(meta$END, "%Y") %in% end.year)
-  meta <- meta[id,]
-  
+  meta <- meta[id, ]
+
   ## approximate distance to site
-  if (!missing(lat) && !missing(lon)) {
-    r <- 6371 # radius of the Earth
-    
-    ## Coordinates need to be in radians
-    meta$longR <- meta$LON * pi / 180
-    meta$latR <- meta$LAT * pi / 180
-    LON <- lon * pi / 180
-    LAT <- lat * pi / 180
-    meta$dist <- acos(sin(LAT) * sin(meta$latR) + cos(LAT) *
-                        cos(meta$latR) * cos(meta$longR - LON)) * r
-    
-    ## sort and retrun top n nearest
-    meta <- utils::head(openair:::sortDataFrame(meta, key = "dist"), n)
+  if (!is.na(lat) && !is.na(lon)) {
+    point <-
+      sf::st_as_sf(
+        data.frame(lon = lon, lat = lat),
+        coords = c("lon", "lat"),
+        crs = sf::st_crs(crs)
+      ) |>
+      sf::st_transform(crs = sf::st_crs(4326))
+
+    meta_sf <-
+      sf::st_as_sf(meta, coords = c("LON", "LAT"), crs = 4326)
+
+    meta$dist <- as.numeric(sf::st_distance(meta_sf, point)) / 1000L
+
+    ## sort and return top n nearest
+    meta <- dplyr::slice_min(meta, order_by = .data$dist, n = n)
   }
-  
-  dat <- rename(meta, latitude = LAT, longitude = LON)
-  
+
+  dat <- dplyr::rename(meta, latitude = "LAT", longitude = "LON")
+
   names(dat) <- tolower(names(dat))
-  
+
   if (plot) {
     content <- paste(
       paste0("<b>", dat$station, "</b>"),
@@ -144,7 +159,7 @@ getMeta <- function(site = "heathrow",
       paste("<b>End:</b>", dat$end),
       sep = "<br/>"
     )
-    
+
     if ("dist" %in% names(dat)) {
       content <- paste(
         content,
@@ -152,27 +167,26 @@ getMeta <- function(site = "heathrow",
         sep = "<br/>"
       )
     }
-    
+
     m <- leaflet::leaflet(dat)
-    
+
     for (i in provider) {
       m <- leaflet::addProviderTiles(map = m, provider = i, group = i)
     }
-    
+
     m <-
       leaflet::addMarkers(
         map = m,
-        ~ longitude,
-        ~ latitude,
+        ~longitude,
+        ~latitude,
         popup = content,
         clusterOptions = leaflet::markerClusterOptions()
       )
-    
+
     if (!is.na(lat) && !is.na(lon)) {
       m <- leaflet::addCircles(
         map = m,
-        lng = lon,
-        lat = lat,
+        data = point,
         weight = 20,
         radius = 200,
         stroke = TRUE,
@@ -185,24 +199,27 @@ getMeta <- function(site = "heathrow",
         )
       )
     }
-    
+
     if (length(provider) > 1) {
-      m <- 
+      m <-
         leaflet::addLayersControl(
           map = m,
-          baseGroups = provider, 
-          options = leaflet::layersControlOptions(collapsed = FALSE, autoZIndex = FALSE)
+          baseGroups = provider,
+          options = leaflet::layersControlOptions(
+            collapsed = FALSE,
+            autoZIndex = FALSE
+          )
         )
     }
-    
+
     print(m)
   }
-  
-  
-  if (returnMap)
+
+  if (returnMap) {
     return(m)
-  else
+  } else {
     return(dat)
+  }
 }
 
 
@@ -214,6 +231,8 @@ getMeta <- function(site = "heathrow",
 #'
 #' @return a [tibble][tibble::tibble-package]
 #'
+#' @family NOAA ISD functions
+#'
 #' @examples
 #' \dontrun{
 #' meta <- getMetaLive()
@@ -222,36 +241,37 @@ getMeta <- function(site = "heathrow",
 #' @export
 getMetaLive <- function(...) {
   ## downloads the whole thing fresh
-  
+
   url <- "https://www1.ncdc.noaa.gov/pub/data/noaa/isd-history.csv"
-  meta <- read_csv(
+  meta <- readr::read_csv(
     url,
     skip = 21,
     col_names = FALSE,
-    col_types = cols(
-      X1 = col_character(),
-      X2 = col_character(),
-      X3 = col_character(),
-      X4 = col_character(),
-      X5 = col_character(),
-      X6 = col_character(),
-      X7 = col_double(),
-      X8 = col_double(),
-      X9 = col_double(),
-      X10 = col_date(format = "%Y%m%d"),
-      X11 = col_date(format = "%Y%m%d")
-    ), 
+    col_types = readr::cols(
+      X1 = readr::col_character(),
+      X2 = readr::col_character(),
+      X3 = readr::col_character(),
+      X4 = readr::col_character(),
+      X5 = readr::col_character(),
+      X6 = readr::col_character(),
+      X7 = readr::col_double(),
+      X8 = readr::col_double(),
+      X9 = readr::col_double(),
+      X10 = readr::col_date(format = "%Y%m%d"),
+      X11 = readr::col_date(format = "%Y%m%d")
+    ),
     progress = FALSE
   )
-  
+
   # if not available e.g. due to US Government shutdown, flag and exit
   # some header data may still be read, so check column number
-  if (ncol(meta) == 1L)
-    stop(
-      "File not available, check \nhttps://www1.ncdc.noaa.gov/pub/data/noaa/ for potential server problems.",
+  if (ncol(meta) == 1L) {
+    cli::cli_abort(
+      "File not available, check {.url https://www1.ncdc.noaa.gov/pub/data/noaa/} for potential server problems.",
       call. = FALSE
     )
-  
+  }
+
   ## names in the meta file
   names(meta) <- c(
     "USAF",
@@ -266,21 +286,13 @@ getMetaLive <- function(...) {
     "BEGIN",
     "END"
   )
-  
+
   ## full character string of site id
   meta$USAF <-
-    formatC(meta$USAF,
-            width = 6,
-            format = "d",
-            flag = "0")
-  
+    formatC(meta$USAF, width = 6, format = "d", flag = "0")
+
   ## code used to query data
   meta$code <- paste0(meta$USAF, "-", meta$WBAN)
-  
+
   return(meta)
 }
-
-# how to update meta data
-# meta <- getMeta(end.year = "all")
-# usethis::use_data(meta, overwrite = TRUE, internal = TRUE)
-# usethis::use_data(meta, overwrite = TRUE)
